"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var registry_1 = require("./registry");
var express = require("express");
var chai_1 = require("chai");
var request = require("supertest");
var js_restful_1 = require("js-restful");
var decorators_1 = require("./decorators");
var descriptions_1 = require("./descriptions");
var util = require("./test-util.spec");
var anyBook = { name: 'simsons' };
var TestService = (function () {
    function TestService() {
    }
    TestService.prototype.doPut = function (req, res) {
        return req.method + (res ? '-response-present' : '');
    };
    TestService.prototype.error = function (req) {
        // simulates an unsuported context type
    };
    TestService.prototype.doNotRenderTheResult = function (res) {
        res.send('manually send');
        return null;
    };
    TestService.prototype.expressContext = function (next) {
        return typeof next;
    };
    TestService.prototype.errorExpress = function (req) {
        // simulates an unsuported expres  context type
    };
    TestService.prototype.withSecContext = function (context) {
        return context.isUserInRole('user');
    };
    __decorate([
        js_restful_1.PUT(),
        __param(0, js_restful_1.Context(js_restful_1.ContextTypes.HttpRequest)), __param(1, js_restful_1.Context(js_restful_1.ContextTypes.HttpResponse)),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Object]),
        __metadata("design:returntype", void 0)
    ], TestService.prototype, "doPut", null);
    __decorate([
        js_restful_1.PUT(),
        js_restful_1.Path('/error'),
        __param(0, js_restful_1.Context(1000)),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TestService.prototype, "error", null);
    __decorate([
        js_restful_1.GET(),
        js_restful_1.Path('/doNotRenderTheResult'),
        __param(0, js_restful_1.Context(js_restful_1.ContextTypes.HttpResponse)),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TestService.prototype, "doNotRenderTheResult", null);
    __decorate([
        js_restful_1.GET(),
        js_restful_1.Path('/expresscontext'),
        __param(0, decorators_1.ExpressContext(descriptions_1.ExpressContextType.HttpNextFunction)),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Function]),
        __metadata("design:returntype", void 0)
    ], TestService.prototype, "expressContext", null);
    __decorate([
        js_restful_1.PUT(),
        js_restful_1.Path('/errorExpress'),
        __param(0, decorators_1.ExpressContext(1000)),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TestService.prototype, "errorExpress", null);
    __decorate([
        js_restful_1.GET(),
        js_restful_1.Path('/withSecContext'),
        __param(0, js_restful_1.SecurityContext()),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], TestService.prototype, "withSecContext", null);
    TestService = __decorate([
        js_restful_1.Path('/books')
    ], TestService);
    return TestService;
}());
describe('service-registry: HTTP methods with Context decorator', function () {
    var app;
    var testService;
    beforeEach(function () {
        app = express();
        testService = new TestService();
        var registry = new registry_1.JsRestfulRegistry(app);
        registry.registerSecurityContextFactory(new util.Factory());
        registry.registerService(testService);
    });
    it('should test a PUT method with a content decorator', function (done) {
        request.agent(app).put('/books').end(function (err, res) {
            chai_1.expect(res.status).to.equal(200);
            chai_1.expect(res.text).to.eql('PUT-response-present');
            done();
        });
    });
    it('should throw an error if an unsupported context type is used', function (done) {
        request.agent(app).put('/books/error').end(function (err, res) {
            chai_1.expect(res.status).to.equal(500);
            chai_1.expect(JSON.stringify(res.error)).to.contains('unsupported contexttype');
            done();
        });
    });
    it('should not render the resut by js-restful-express - this should be done by the service method', function (done) {
        request.agent(app).get('/books/doNotRenderTheResult').end(function (err, res) {
            chai_1.expect(res.status).to.equal(200);
            chai_1.expect(res.text).to.contains('manually send');
            done();
        });
    });
    it('should test a method with express context type next function', function (done) {
        request.agent(app).get('/books/expresscontext').end(function (err, res) {
            chai_1.expect(res.status).to.equal(200);
            chai_1.expect(res.text).to.contains('function');
            done();
        });
    });
    it('should throw an error if an unsupported express context type is used', function (done) {
        request.agent(app).put('/books/errorExpress').end(function (err, res) {
            chai_1.expect(res.status).to.equal(500);
            chai_1.expect(JSON.stringify(res.error)).to.contains('unsupported contexttype');
            done();
        });
    });
    it('should test a method with a security context', function (done) {
        request.agent(app).get('/books/withSecContext').end(function (err, res) {
            chai_1.expect(res.status).to.equal(200);
            chai_1.expect(res.text).to.contains('true');
            done();
        });
    });
});
//# sourceMappingURL=registry-context.spec.js.map